<?php

namespace App\Service;

use App\Exception;

class OrderService {

    private $util;
    private $orderFactory;
    private $productService;
    private $paymentService;

    public function __construct (\App\Util\Util $util,
                                 \App\Service\ProductService $productService,
                                 \App\Service\PaymentService $paymentService,
                                 \App\Factory\OrderFactory $orderFactory) {
        $this->util = $util;
        $this->productService  = $productService;
        $this->paymentService  = $paymentService;
        $this->orderFactory    = $orderFactory;
    }
    
    public function checkParams($params){
        
        $generalParams = [];
        $generalParams['NMFILIAL'] = $this->orderFactory->getFilial($params['subsidiaryCode']);
        if(empty($generalParams['NMFILIAL']))
            throw new Exception\ParamsException("Filial {$params['subsidiaryCode']} não encontrada.");
        
        $dtInicial = empty($params['dtInicial']) ? date("d-m-Y", strtotime('-1 day')) : str_replace("/", "-", $params['dtInicial']);
        $dtFinal   = empty($params['dtFinal'])   ? date("d-m-Y", strtotime('-1 day')) : str_replace("/", "-", $params['dtFinal']);
        
        $dtInicial = new \DateTime($dtInicial);
        $dtFinal = new \DateTime($dtFinal);

        $diff  = date_diff($dtInicial,$dtFinal);
        if($diff->d > 5 || $diff->m > 0 ) 
            throw new Exception\ParamsException("Intervalo de datas superior a 5 dias ou data final inválida.");
        
        $dtInicial = date_format($dtInicial,'d-m-Y');
        $dtFinal = date_format($dtFinal,'d-m-Y');
            
        if(strtotime($dtInicial) > strtotime($dtFinal))
            throw new Exception\ParamsException("Data inicial superior a data final ou inválida.");
        
        $generalParams['DTINICIAL'] = $dtInicial;
        $generalParams['DTFINAL'] = $dtFinal;
            
        return $generalParams;
    }
    
    public function getSales($params){
        $generalParams = $params;
        
        $branch = $this->checkParams($generalParams);
        $subsidiaryCode = $generalParams['subsidiaryCode'];
        $orders = $this->orderFactory->getSales($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);

        $salesGroup = [];
        
        $salesGroup = array(
            'subsidiaryCode' => $subsidiaryCode,
            'subsidiaryName' => $branch['NMFILIAL'],
            'sales'          => [],
            'unutilized'     => [],
            );

        foreach($orders as $order){
            
            $vrTotVenda = $this->orderFactory->getVrTotalVenda($order['CDFILIAL'], $order['CDCAIXA'] , $order['NRSEQVENDA']);
            $vrDescAcrescTotal = $this->orderFactory->getTotalAcreDesc($order['CDFILIAL'], $order['CDCAIXA'] , $order['NRSEQVENDA']);
            $vrTotalImposto = $this->orderFactory->getTotalImpostos($order['CDFILIAL'], $order['CDCAIXA'] , $order['NRSEQVENDA']);

            $sale  = [
                'noteId'              => $order['NRSEQVENDA'],
                'noteNumber'          => $order['NRLANCTONFCE'],
                'noteSeries'          => $order['CDSERIENFCE'],
                'chaveSefaz'          => $order['NRACESSONFCE'],
                'protocSefaz'         => $order['NRPROTOCOLONFCE'],
                'cupomNumber'         => $order['NRNOTAFISCALCE'],
                'subsidiaryCode'      => $order['CDFILIAL'],
                'emissionDate'        => $order['DTEMISSAONFCE'],
                'saleDate'            => $order['DTVENDA'],
                'saleStatus'          => $this->getSituVenda($order['IDSITUVENDA']),
                'noteStatus'          => $order['IDSTATUSNFCE'],
                'protoCancSefaz'      => $order['NRPROTOCOLOCANC'],
                'tableId'             => !empty($order['NRMESA']) ? $order['NRMESA'] : '',
                'quantityPeople'      => !empty($order['NRPESMESAVENDA']) ? $order['NRPESMESAVENDA'] : '1',
                'motInutiliza'        => null,
                'saleTotalAmount'     => isset($order['VRTOTVENDA']) ? floatval($this->util->tofloat($order['VRTOTVENDA'])) : 0,
                'noteTotalAmount'     => isset($vrTotVenda['VRTOTAL']) ? floatval($this->util->tofloat($vrTotVenda['VRTOTAL'])) : 0,
                'totalDiscountAmount' => floatval($this->util->tofloat($vrDescAcrescTotal['desconto'])),
                'totalAmountAddition' => floatval($this->util->tofloat($vrDescAcrescTotal['acrescimo'])),
                'totalAmountPIS'      => floatval($this->util->tofloat($vrTotalImposto['vrPIS'])),
                'totalAmountCOFINS'   => floatval($this->util->tofloat($vrTotalImposto['vrCOFINS'])),
                'totalAmountICMS'     => floatval($this->util->tofloat($vrTotalImposto['vrICMS'])),
                'payments'            => [],
                'itens'               => []

                ];
                
            $itens = $this->productService->getItensVenda($order['CDFILIAL'], $order['CDCAIXA'] , $order['NRSEQVENDA']);
            array_push($sale['itens'],$itens);
            
            $payments = $this->paymentService->getPayments($order['CDFILIAL'], $order['CDCAIXA'] , $order['NRSEQVENDA']);
            array_push($sale['payments'],$payments);
            array_push($salesGroup['sales'], $sale);
        }
        
        $notasInutilizadas = $this->orderFactory->getUnutilizedNotes($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        if(!empty($notasInutilizadas)) {
            $arrayunutilized = [];
            foreach($notasInutilizadas as $nota){
                $unutilized = [
                        'subsidiaryCode'           =>  $nota['CDFILIAL'],
                        'cupomNumber'              =>  $nota['NRNOTAFISCALCE'],
                        'noteSeries'               =>  $nota['CDSERIENFCE'],
                        'noteStatus'               =>  $nota['IDSTATUSNFCE'],
                        'noteDate'                 =>  $nota['DTENTRVENDA'],
                        'inutilizationDate'        =>  $nota['DTHRINUTNFCE'],
                        'inutilizationDescription' =>  $nota['DSOBSINUTNFCE']
                    ];
                    
                array_push($arrayunutilized,$unutilized);
            }
            array_push($salesGroup['unutilized'], json_decode(json_encode($arrayunutilized), true));
        }
        return $salesGroup;
    }
    
    public function getSituVenda($idSituVenda){
        $statusVenda = array(
            'O' => 'Concluida',
            'C' => 'Cancelada'
            );
            
        return isset($statusVenda[$idSituVenda]) ? $statusVenda[$idSituVenda] : null;
    }
    
    public function getReduction($params){
        $generalParams = $params;
        
        $branch = $this->checkParams($generalParams);
        $subsidiaryCode = $generalParams['subsidiaryCode'];
        
        $closingType = $this->orderFactory->getClosingType($subsidiaryCode);

        if($closingType['IDTPAPUFIL'] === 'S'){
            $cdTpSangria = $this->orderFactory->getTipoSangria($subsidiaryCode);
            
            if(empty($cdTpSangria))
                throw new Exception\ParamsException("Tipo de sangria de fechamento não parametrizado para a filial {$subsidiaryCode}.");
                
            $sangrias = $this->orderFactory->closingPosBySangria($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL'], $cdTpSangria);
        }else{
            $sangrias = $this->orderFactory->closingPosByVenda($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
            
        }
        if(empty($sangrias))
                throw new Exception\ParamsException("Não a dados a serem listados.");
            
        $arraySangrias = [
                    'subsidiaryCode' => $subsidiaryCode,
                    'subsidiaryName' => $branch['NMFILIAL'],
                    'closing'    => []
        ];
            
        foreach($sangrias as $sangria){

            $paymentDetails = $this->orderFactory->getPaymentInfo($sangria['CDTIPORECE']);
            
            if($paymentDetails['IDTIPORECE'] == "1" || $paymentDetails['IDTIPORECE'] == "2"){
                $cardType = $paymentDetails['IDTIPORECE'];
            }elseif($paymentDetails['IDTIPORECE'] == 'G' || $paymentDetails['IDTIPORECE'] == 'F'){
                $cardType = "3";
            }else{
                $cardType = null;
            }
                
            if(empty($cardType))
                $cardDescription = $this->paymentService->getCardType($cardType);
            
            $receipt = array(
                    'movimentDate'    => str_replace("-", "/",$sangria['DTABERCAIX']),
                    'payment'         => $paymentDetails['NMTIPORECE'],
                    'cardType'        => !empty($cardType) ? $cardType : '',
                    'cardDescription' => !empty($cardType) ? $cardDescription : '',
                    'paymentCode'     => $paymentDetails['CDTIPORECE'],
                    'externalCode'    => $paymentDetails['CDTPRECEINTE'],
                    'amount'          => floatval($this->util->tofloat($sangria['VRMOVIVEND']))
                    );
            array_push($arraySangrias['closing'], $receipt);
        }   
        return $arraySangrias;
        
    }
    
    public function getStockExit($params) {
        $generalParams = $params;
        $branch = $this->checkParams($generalParams);
        
        $subsidiaryCode = $generalParams['subsidiaryCode'];
        $stock = [
                'subsidiaryCode' => $subsidiaryCode,
                'subsidiaryName' => $branch['NMFILIAL'],
                'stockRequest'   => []
            
            ];
            
        $nrLancEstq = [];
        
        $planRequests = $this->orderFactory->getPlanRequest($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);

        if(!empty($planRequests)){
            array_push($nrLancEstq, $planRequests);
        }
        
        $saleRequests = $this->orderFactory->getSaleRequest($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        if(!empty($saleRequests)){
            array_push($nrLancEstq, $saleRequests);
        }

        $movEst = [];
        
        foreach($nrLancEstq as $modLanc) {
            foreach($modLanc as $lanc) {
            
                $arrayItens = $this->orderFactory->getMovEstoq($subsidiaryCode, $lanc['NRLANCESTQ']);
                $reqEst = array (
                            'requisitionId' => $lanc['NRLANCESTQ'],
                            'date'          => $this->orderFactory->getDateMovEst($subsidiaryCode, $lanc['NRLANCESTQ']),
                            'itens'         => []
                        );
    
                foreach($arrayItens as $itens) {
                    $typeItem = $this->orderFactory->getTypeItem($itens['CDPRODUTO']);
                    if (empty($itens['QTSUBPRODLOC']) && empty($itens['QTSUBPRODROT']) && empty($itens['QTPRODVEND'])) {
                        $qtd = $this->util->tofloatQtd($itens['QTLANCTOEST']);
                    } else if (empty($itens['QTSUBPRODLOC']) && empty($itens['QTSUBPRODROT']) && !empty($itens['QTPRODVEND'])) {
                        $qtd = $this->util->tofloatQtd($itens['QTPRODVEND']);
                    } else {
                        if(empty($itens['QTSUBPRODLOC']) && !empty(['QTDCADCOMPLOCAL'])) {
                            continue;
                        }
                        
                        if(empty($itens['QTSUBPRODLOC']) && empty($itens['QTDCADCOMPLOCAL']) && !empty($itens['QTSUBPRODROT'])) {
                            $qtdComp = $this->util->tofloatQtd($itens['QTSUBPRODROT']);
                        } else if (!empty($itens['QTSUBPRODLOC'])) {
                            $qtdComp = $this->util->tofloatQtd($itens['QTSUBPRODLOC']);
                        }
                        
                        $qtdVend = $this->util->tofloatQtd($itens['QTPRODVEND']);
                        $qtd = $this->util->tofloatQtd(bcmul($qtdComp, $qtdVend, 4));
                    }
                    $qtd = round($qtd, 4);
                    $itemDetails = array(
                                    'requisitionId'       => $itens['NRLANCESTQ'],
                                    'cupomNumber'         => !empty($itens['NRNOTAFISCALCE']) ? $itens['NRNOTAFISCALCE'] : '',
                                    'name'                => $itens['NMPRODUTO'],
                                    'code'                => $itens['CDARVPROD'],
                                    'externalCode'        => $itens['CDPRODINTE'],
                                    'stockEvolved'        => $itens['IDEVOLESTOQ'] == 'S'? true : false, 
                                    'stockWithdrawalType' => "{$itens['CDTIPORETI']} - {$itens['NMTIPORETI']}",
                                    'idTypeItem'          => $typeItem['CDTPITEMPROD'],
                                    'TypeItemDescription' => $typeItem['NMTPITEMPROD'],
                                    'unit'                => $itens['SGUNIDADE'],
                                    'quantity'            => $qtd
                                 );
                    array_push($reqEst['itens'], $itemDetails);
                }
                array_push($movEst, $reqEst);
            }
        }
        
        if(!empty($movEst))
            $stock['stockRequest'] = $movEst;
        
        return $stock;
        
    }
    
    private function recalculateQtd($itens) {
        if ((empty($itens['QTDCOMP'])) || (empty($itens['QTPRODVEND']))) {
            return $itens['QTLANCTOEST'];
        } else {
            $qtdComp = $this->util->tofloatQtd($itens['QTDCOMP']);
            $qtdVend = $this->util->tofloatQtd($itens['QTPRODVEND']);
            return bcmul($qtdComp, $qtdVend, 4);
        }
        
    }
    
    public function getStockExitFull($params) {
        $generalParams = $params;
        $branch = $this->checkParams($generalParams);
        
        $subsidiaryCode = $generalParams['subsidiaryCode'];
        $stock = [
                'subsidiaryCode' => $subsidiaryCode,
                'subsidiaryName' => $branch['NMFILIAL'],
                'stockRequest'   => []
            
            ];
            
/*
        ================================================================================================================================
        
                    Rotina de consulta de saìdas de estoque da consolidação de vendas com base em composição de produto
        
        ================================================================================================================================
        
        $nrLancEstq = [];
        
        $saleRequests = $this->orderFactory->getSaleRequest($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        if(!empty($saleRequests)){
            array_push($nrLancEstq, $saleRequests);
        }
            
        $movEst = [];
        foreach($nrLancEstq[0] as $modLanc) {
                $arrayItens = $this->orderFactory->getMovEstoq($subsidiaryCode, $modLanc['NRLANCESTQ']);
                $reqEst = array (
                            'requisitionId' => $modLanc['NRLANCESTQ'],
                            'date'          => $this->orderFactory->getDateMovEst($subsidiaryCode, $modLanc['NRLANCESTQ']),
                            'itens'         => []
                        );
                foreach($arrayItens as $itens) {
                    $typeItem = $this->orderFactory->getTypeItem($itens['CDPRODUTO']);
                    if (empty($itens['QTSUBPRODLOC']) && empty($itens['QTSUBPRODROT']) && empty($itens['QTPRODVEND'])) {
                        $qtd = $this->util->tofloatQtd($itens['QTLANCTOEST']);
                    } else if (empty($itens['QTSUBPRODLOC']) && empty($itens['QTSUBPRODROT']) && !empty($itens['QTPRODVEND'])) {
                        $qtd = $this->util->tofloatQtd($itens['QTPRODVEND']);
                    } else {
                        if(empty($itens['QTSUBPRODLOC']) && !empty(['QTDCADCOMPLOCAL'])) {
                            continue;
                        }
                        
                        if(empty($itens['QTSUBPRODLOC']) && empty($itens['QTDCADCOMPLOCAL']) && !empty($itens['QTSUBPRODROT'])) {
                            $qtdComp = $this->util->tofloatQtd($itens['QTSUBPRODROT']);
                        } else if (!empty($itens['QTSUBPRODLOC'])) {
                            $qtdComp = $this->util->tofloatQtd($itens['QTSUBPRODLOC']);
                        }
                        
                        $qtdVend = $this->util->tofloatQtd($itens['QTPRODVEND']);
                        $qtd = $this->util->tofloatQtd(bcmul($qtdComp, $qtdVend, 4));
                    }
                    $qtd = round($qtd, 4);
                    $itemDetails = array(
                                    'requisitionId'        => $itens['NRLANCESTQ'],
                                    'cupomNumber'          => !empty($itens['NRNOTAFISCALCE']) ? $itens['NRNOTAFISCALCE'] : '',
                                    'name'                 => $itens['NMPRODUTO'],
                                    'code'                 => $itens['CDARVPROD'],
                                    'externalCode'         => $itens['CDPRODINTE'],
                                    'stockEvolved'         => $itens['IDEVOLESTOQ'] == 'S'? true : false, 
                                    'stockWithdrawalType'  => "{$itens['CDTIPORETI']} - {$itens['NMTIPORETI']}",
                                    'idTypeItem'           => $typeItem['CDTPITEMPROD'],
                                    'TypeItemDescription'  => $typeItem['NMTPITEMPROD'],
                                    'storeRoomCode'        => $itens['CDALMOXARIFE'],
                                    'storeRoomDescription' => $itens['DSALMOXARIFE'],
                                    'unit'                 => $itens['SGUNIDADE'],
                                    'quantity'             => $qtd
                                 );
                    array_push($reqEst['itens'], $itemDetails);
                }
                array_push($movEst, $reqEst);
        }
        
        unset($itemDetails);
        unset($reqEst);
*/
        
        $generalStock = $this->orderFactory->getGeneralStock($subsidiaryCode, $branch['DTINICIAL'], $branch['DTFINAL']);
        $arrGeneralStock = [];
        if(!empty($generalStock)) {
            
            $reqEst = array (
                            'requisitionId' => '',
                            'description'   => '',
                            'date'          => '',
                            'itens'         => []
                    );
            foreach($generalStock as $itens) {
                    if($itens['NRLANCESTQ'] != $reqEst['requisitionId']) {
                        if(!empty($reqEst['requisitionId'])) {
                            array_push($arrGeneralStock, $reqEst);
                        }
                        $reqEst['requisitionId'] = $itens['NRLANCESTQ'];
                        $reqEst['description']   = !empty($itens['DSLANCESTQ']) ? $itens['DSLANCESTQ'] : '';
                        $reqEst['date']          = $itens['DTLANCMOVI'];
                        $reqEst['itens']         = [];
                    }
                    
                    $typeItem = $this->orderFactory->getTypeItem($itens['CDPRODUTO']);
                    $qtd = $this->util->tofloatQtd($itens['QTLANCTOEST']);
                    $qtd = round($qtd, 4);
                    $itemDetails = array(
                                    'requisitionId'        => $itens['NRLANCESTQ'],
                                    'name'                 => $itens['NMPRODUTO'],
                                    'code'                 => $itens['CDARVPROD'],
                                    'externalCode'         => $itens['CDPRODINTE'],
                                    'stockEvolved'         => $itens['IDEVOLESTOQ'] == 'S' ? true : false,
                                    'stockWithdrawalType'  => "{$itens['CDTIPORETI']} - {$itens['NMTIPORETI']}",
                                    'costCenterId'         => !empty($itens['CDCENTCUST']) ? $itens['CDCENTCUST'] : '',
                                    'costCenter'           => !empty($itens['NMCENTCUST']) ? $itens['NMCENTCUST'] : '',
                                    'idTypeItem'           => $typeItem['CDTPITEMPROD'],
                                    'TypeItemDescription'  => $typeItem['NMTPITEMPROD'],
                                    'storeRoomCode'        => !empty(trim($itens['CDALMOXARIFE'])) ? $itens['CDALMOXARIFE'] : '',
                                    'storeRoomDescription' => !empty(trim($itens['DSALMOXARIFE'])) ? $itens['DSALMOXARIFE'] : '', 
                                    'unit'                 => $itens['SGUNIDADE'],
                                    'quantity'             => $this->roundValue($itens['QTLANCTOEST'])
                                 );
                                 
                    array_push($reqEst['itens'], $itemDetails);
                }
                 array_push($arrGeneralStock, $reqEst);
        }
            $stock['stockRequest'] = $arrGeneralStock;
        
        return $stock;
        
    }
    
    private function roundValue($value) {
        $value = $this->util->tofloatQtd($value);
        return round($value, 3);
    }
    
}